﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

using System.Data;

using Persistence;
using doLogic;

public partial class _Default : System.Web.UI.Page
{
	Client _client = null;

    protected void Page_Load(object sender, EventArgs e)
    {
        //create a new client on first run
        Client client = null;
        if (Clients.All.Count == 0)
        {
            client = new Client();
            client.ClientName = "Test Client, Sole Proprietor.";
            client.ClientShortName = "Test";
            client.Store();
            Clients.All.Add(client);
        }

        client = Clients.All.ItemById(1);


        lblMessage.Text = "";

        if (Session["ClientCode"].IsEmpty())
            Session["ClientCode"] = "1";

		if (Request.QueryString["action"] == "new")
		{
			_client = new Client();
			btnNewClient.Visible = false;
			gvClient.Visible = false;
			tblDivisionsAndLocations.Visible = false;
			pnlManagers.Visible = false;
			pnlEditClient.Visible = true;
			txtClientName.Focus();
            Session["ClientName"] = "New Client";
		}
		else
		{
			_client = Clients.All.ItemById(Session["ClientCode"].ToInt32());
			btnNewClient.Visible = true;
			gvClient.Visible = true;
			tblDivisionsAndLocations.Visible = true;
			pnlManagers.Visible = true;
            Session["ClientName"] = _client.ClientLongName;
		}
		
		if (!IsPostBack)
        {
            BindClient();
            BindDivisions();
            BindLocations();
            BindManagers();
        }
    }

    #region Bindings

    private void BindClient()
    {
		gvClient.DataSource = new List<Client> { _client };
        gvClient.DataBind();
    }

    private void BindDivisions()
    {
        BindDivisions(SortDirection.Ascending);
    }

    private void BindDivisions(SortDirection sortDirection)
    {
        Divisions divisions = Divisions.For(_client);
        
        if (txtSearchDivisions.Text.IsNotEmpty())
            divisions = divisions.WhereNameContains(txtSearchDivisions.Text);
        
        if (divisions.Count == 1)
            gridDivisions.SelectedIndex = 0;

        if (sortDirection == SortDirection.Ascending)
            gridDivisions.DataSource = divisions.OrderBy(d => d.DivName).ToList();
        else
            gridDivisions.DataSource = divisions.OrderByDescending(d => d.DivName).ToList();
        gridDivisions.DataBind();

        if (ddlDivisions.Items.Count == 0)
            ddlDivisions.BindList(Divisions.For(_client), "DivName", "DivID");
        ddlDivisions.SelectItem("", "");

        if (ddlAssignDivision.Items.Count == 0)
            ddlAssignDivision.BindList(Divisions.For(_client), "DivName", "DivID");
        ddlAssignDivision.SelectItem("", "");
    }

    private void BindLocations()
    {
        BindLocations(SortDirection.Ascending, "LocationName");
    }

    private void BindLocations(SortDirection sortDirection, string sortExpression)
    {
        int divisionID = 0;
        if (gridDivisions.SelectedDataKey != null)
            divisionID = gridDivisions.SelectedDataKey.Value.ToInt32();

        Locations locations = Locations.For(_client);

        if (ddlAssignLocation.Items.Count == 0)
            ddlAssignLocation.BindList(locations, "LocationName", "LocID");
        ddlAssignLocation.SelectItem("", "");

		if (divisionID.IsNotEmpty())
			locations = locations.ForDivision(divisionID);

        if (txtSearchLocations.Text.IsNotEmpty())
            locations = locations.WhereNameContains(txtSearchLocations.Text);

        if (locations.Count == 1)
            gridLocations.SelectedIndex = 0;

        if (sortDirection == SortDirection.Ascending)
            if (sortExpression == "LocationName")
                gridLocations.DataSource = locations.OrderBy(d => d.LocationName).ToList();
            else
                gridLocations.DataSource = locations.OrderBy(d => d.DivisionName).ToList();
        else
            if (sortExpression == "LocationName")
                gridLocations.DataSource = locations.OrderByDescending(d => d.LocationName).ToList();
            else
                gridLocations.DataSource = locations.OrderByDescending(d => d.DivisionName).ToList();

        gridLocations.DataBind();
    }

    private void BindManagers()
    {
        if (rdoDivisions.Checked && gridDivisions.SelectedDataKey != null)
        {
            gridManagers.DataSource = DivisionManagers.For(gridDivisions.SelectedDataKey.Value.ToInt32()).Matching(txtSearchManagers.Text);
            gridManagers.EmptyDataText = String.Format("No managers assigned to division: {0}.", gridDivisions.SelectedRow.GetValueIn("txtDivisionName"));
            //gridManagers.GetColumn("Primary").Visible = true;
        }
        else if (rdoLocations.Checked && gridLocations.SelectedDataKey != null)
        {
            gridManagers.DataSource = LocationManagers.For(gridLocations.SelectedDataKey.Value.ToInt32()).Matching(txtSearchManagers.Text);
            gridManagers.EmptyDataText = String.Format("No managers assigned to location: {0}.", gridLocations.SelectedRow.GetValueIn("txtLocationName"));
            //gridManagers.GetColumn("Primary").Visible = true;
        }
        else if (rdoClients.Checked)
        {
            gridManagers.DataSource = Managers.For(Session["ClientCode"].ToInt32()).Matching(txtSearchManagers.Text);
            gridManagers.EmptyDataText = String.Format("No managers assigned to {0}.", Session["ClientName"]);
            //gridManagers.GetColumn("Primary").Visible = false;
        }
        else
            gridManagers.EmptyDataText = "";

        gridManagers.DataBind();

        if (gridDivisions.SelectedDataKey != null)
            rdoDivisions.Text = "Division: " + gridDivisions.SelectedRow.GetValueIn("txtDivisionName");
        else
            rdoDivisions.Text = "Division";

        if (gridLocations.SelectedDataKey != null)
            rdoLocations.Text = "Location: " + gridLocations.SelectedRow.GetValueIn("txtLocationName");
        else
            rdoLocations.Text = "Location";

        BindManager();
        BindAssignedDivisions();
        BindAssignedLocations();
    }

    private void BindManager()
    {
        List<Manager> list = null;
        if (gridManagers.SelectedDataKey != null)
            list = new List<Manager> { new Manager(gridManagers.SelectedDataKey.Value.ToInt32()) };

        dvManager.DataSource = list;
        dvManager.DataBind();

        btnDeleteManager.Visible = (gridManagers.SelectedIndex >= 0);
    }

    private void BindAssignedLocations()
    {
        ManagerLocations list = null;
        if (gridManagers.SelectedDataKey != null)
            list = ManagerLocations.For(gridManagers.SelectedDataKey.Value.ToInt32());

        gvAssignedLocations.DataSource = list;
        gvAssignedLocations.DataBind();

        DropDownList ddl = dvManager.FindControl("ddlAssignedLocations") as DropDownList;
        if (ddl != null)
            ddl.BindList(list, "LocationName", "LocationID");
    }

    private void BindAssignedDivisions()
    {
        ManagerDivisions list = null;
        if (gridManagers.SelectedDataKey != null)
            list = ManagerDivisions.For(gridManagers.SelectedDataKey.Value.ToInt32());

        gvAssignedDivisions.DataSource = list;
        gvAssignedDivisions.DataBind();

        DropDownList ddl = dvManager.FindControl("ddlAssignedDivisions") as DropDownList;
        if (ddl != null)
            ddl.BindList(list, "DivisionName", "DivisionID");
    }

    #endregion

    #region gvClient

    protected void gvClient_RowDataBound(object sender, GridViewRowEventArgs e)
    {
        if (e.Row.RowType == DataControlRowType.DataRow)
        {
            if ((e.Row.RowState & DataControlRowState.Edit) == DataControlRowState.Edit)
            {
            }
            else
            {
                for (int i = 0; i < e.Row.Cells.Count; i++)
                    e.Row.Cells[i].Attributes["onclick"] = this.Page.ClientScript.GetPostBackClientHyperlink((GridView)sender, "Select$" + e.Row.RowIndex);
            }

            if (((GridView)sender).EditIndex < 0)
            {
                e.Row.Attributes["onmouseover"] = "if (this.className != 'selectedrow') this.className='mouseover'";
                e.Row.Attributes["onmouseout"] = "if (this.className != 'selectedrow') this.className='mouseout'";
            }
        }
    }

    protected void gvClient_RowCommand(object sender, GridViewCommandEventArgs e)
    {
        if (e.CommandName == "Select")
        {
            if (pnlEditClient.Visible)
            {
                pnlEditClient.Visible = false;
                return;
            }

            //bind and show edit panel
            gvClient.SelectedIndex = e.CommandArgument.ToInt32();
            doLogic.Client c = doLogic.Clients.All.ItemById(gvClient.SelectedDataKey["ClientCode"].ToInt32());
            chkActive.Checked = c.Active;
            txtClientName.Text = c.ClientName;
            txtClientShortName.Text = c.ClientShortName;
            txtBillingAddress.Text = c.BillingAddress;
            txtCity.Text = c.City;
            txtState.Text = c.State;
            txtZip.Text = c.Zip;
            txtContactTitle.Text = c.ContactTitle;
            txtContactFirstName.Text = c.ContactFirstName;
            txtContactLastName.Text = c.ContactLastName;
            txtPhoneNumber.Text = c.PhoneNumber.ToPhoneFormat();
            txtExtension.Text = c.Extension;
            txtFaxNumber.Text = c.FaxNumber.ToPhoneFormat();
            txtEmailAddress.Text = c.EmailAddress;
            txtURL.Text = c.URL;
            txtCallCenterPhone.Text = c.CallCenterPhone.ToPhoneFormat();
            txtCallCenterEmail.Text = c.CallCenterEmail;
			chkPayType.Checked = c.WPPayType;
			chkClass.Checked = c.WPClass;
			chkCategory.Checked = c.WPCategory;
			chkUCode.Checked = c.WPUCode;
            pnlEditClient.Visible = true;
        }
    }

    protected void btnSaveClient_Click(object sender, EventArgs e)
    {
		try
		{

			Client c = _client;

			c.Active = chkActive.Checked;
			c.ClientName = txtClientName.Text;
			c.ClientShortName = txtClientShortName.Text;
			c.BillingAddress = txtBillingAddress.Text;
			c.City = txtCity.Text;
			c.State = txtState.Text;
			c.Zip = txtZip.Text;
			c.ContactTitle = txtContactTitle.Text;
			c.ContactFirstName = txtContactFirstName.Text;
			c.ContactLastName = txtContactLastName.Text;
			c.PhoneNumber = txtPhoneNumber.Text.ToNumbersOnly();
			c.Extension = txtExtension.Text.ToNumbersOnly();
			c.FaxNumber = txtFaxNumber.Text.ToNumbersOnly();
			c.EmailAddress = txtEmailAddress.Text;
			c.URL = txtURL.Text;
			c.CallCenterPhone = txtCallCenterPhone.Text.ToNumbersOnly();
			c.CallCenterEmail = txtCallCenterEmail.Text;
			c.WPPayType = chkPayType.Checked;
			c.WPClass = chkClass.Checked;
			c.WPCategory = chkCategory.Checked;
			c.WPUCode = chkUCode.Checked;
			c.Store();

            Session["ClientCode"] = c.ClientCode.ToString();
			Session["ClientName"] = c.ClientLongName;

            if (Request.QueryString["action"] == "new")
            {
                Clients.All.Add(c);
                Response.Redirect("Default.aspx");
            }

			BindClient();
			pnlEditClient.Visible = false;
		}
		catch (System.Data.Common.DbException de)
		{
			if (de.Message.ToLower().Contains("duplicate"))
				ShowMessage("It looks like that client already exists");
			else
				ShowMessage(de.Message);
			//Log.Add(de, Profile.UserName);
		}
		catch (Exception ex)
		{
			ShowMessage(ex.Message);
			//Log.Add(ex, Profile.UserName);
		}
    }

    protected void btnCancelClient_Click(object sender, EventArgs e)
    {
		if (Request.QueryString["action"] == "new")
			Response.Redirect("Default.aspx");
		else
			Session["ClientName"] = Clients.All.ItemById(Session["ClientCode"].ToInt32()).ClientLongName;

		pnlEditClient.Visible = false;
    }

    #endregion

    #region gridDivisions

    protected void btnSearchDivisions_Click(object sender, EventArgs e)
    {
        rdoDivisions.Checked = true;
        rdoLocations.Checked = false;
        rdoClients.Checked = false;
        BindDivisions();
        BindLocations();
        BindManagers();
    }

    protected void gridDivisions_PageIndexChanging(object sender, GridViewPageEventArgs e)
    {
        GridView grid = (GridView)sender;
        grid.PageIndex = e.NewPageIndex;
        BindDivisions();
        BindLocations();
        BindManagers();
    }

    protected void gridDivisions_RowEditing(object sender, GridViewEditEventArgs e)
    {
        GridView grid = (GridView)sender;
        grid.EditIndex = e.NewEditIndex;
        if (grid.EditIndex == grid.SelectedIndex)
            BindDivisions();
        else
        {
            grid.SelectedIndex = grid.EditIndex;
            BindDivisions();
            BindLocations();
            BindManagers();
        }
        grid.SelectedRow.FocusAndSelect("txtDivisionName");
    }

    protected void gridDivisions_Sorting(object sender, GridViewSortEventArgs e)
    {
        BindDivisions(gridDivisions.GetSortDirection());
        BindLocations();
        BindManagers();
    }

    protected void gridDivisions_RowCancelingEdit(object sender, GridViewCancelEditEventArgs e)
    {
        GridView grid = (GridView)sender;
        grid.EditIndex = -1;
        BindDivisions();
    }

    protected void gridDivisions_RowDeleting(object sender, GridViewDeleteEventArgs e)
    {
		try
		{
			GridView grid = (GridView)sender;

			Division division = new Division();

			division.DivID = grid.DataKeys[e.RowIndex].Value.ToInt32();
			division.Delete();
			BindDivisions();
			BindLocations();
			BindManagers();
		}
		catch (Exception ex)
		{
			ShowMessage(ex.Message);
			//Log.Add(ex, Profile.UserName);
		}
	}

    protected void gridDivisions_RowUpdating(object sender, GridViewUpdateEventArgs e)
    {
        GridView grid = (GridView)sender;
        GridViewRow row = grid.Rows[e.RowIndex];

        Division division = new Division(grid.DataKeys[e.RowIndex].Value.ToInt32());
        division.DivName = row.GetValueIn("txtDivisionName");
        division.Store();
        grid.EditIndex = -1;
        BindDivisions();
        BindLocations();
        BindManagers();
    }

    protected void btnAddDivision_Click(object sender, EventArgs e)
    {
		try
		{
			Division division = new Division();
			division.ClientCode = Session["ClientCode"].ToInt32();
			division.DivName = txtAddDivName.Text;
			division.Store();

			//reset form
			gridDivisions.EditIndex = -1;
			txtSearchDivisions.Text = "";
			txtAddDivName.Text = "";
			ddlDivisions.Items.Clear();

			BindDivisions();

			gridDivisions.SelectPage(division.DivID.ToString());
			BindLocations();
			BindManagers();
		}
		catch (System.Data.Common.DbException de)
		{
			if (de.Message.ToLower().Contains("duplicate"))
				ShowMessage("That division already exists");
			else
				ShowMessage(de.Message);
			//Log.Add(de, Profile.UserName);
		}
		catch (Exception ex)
		{
			ShowMessage(ex.Message);
			//Log.Add(ex, Profile.UserName);
		}
    }

    protected void gridDivisions_RowDataBound(object sender, GridViewRowEventArgs e)
    {
        if (e.Row.RowType == DataControlRowType.DataRow)
        {
            if ((e.Row.RowState & DataControlRowState.Edit) == DataControlRowState.Edit)
            {
            }
            else
            {
                for (int i = 0; i < e.Row.Cells.Count; i++)
                    e.Row.Cells[i].Attributes["onclick"] = this.Page.ClientScript.GetPostBackClientHyperlink((GridView)sender, "Select$" + e.Row.RowIndex);
            }

            if (((GridView)sender).EditIndex < 0)
            {
                e.Row.Attributes["onmouseover"] = "if (this.className != 'selectedrow') this.className='mouseover'";
                e.Row.Attributes["onmouseout"] = "if (this.className != 'selectedrow') this.className='mouseout'";
            }
        }
    }

    protected void gridDivisions_SelectedIndexChanging(object sender, GridViewSelectEventArgs e)
    {
        rdoClients.Checked = false;
        rdoLocations.Checked = false;
        rdoDivisions.Checked = true;

        GridView grid = (GridView)sender;
        GridViewRow row = grid.Rows[e.NewSelectedIndex];
        grid.SelectedIndex = e.NewSelectedIndex;
        BindLocations();
        BindManagers();
    }

    protected void btnShowAllDivisions_Click(object sender, EventArgs e)
    {
        txtSearchDivisions.Text = "";
        BindDivisions();
        BindLocations();
        BindManagers();
    }

    #endregion

    #region gridLocations

    protected void btnSearchLocations_Click(object sender, EventArgs e)
    {
        rdoDivisions.Checked = false;
        rdoLocations.Checked = true;
        rdoClients.Checked = false;
        gridLocations.SelectedIndex = -1;
        BindLocations();
        BindManagers();
    }

    protected void gridLocations_PageIndexChanging(object sender, GridViewPageEventArgs e)
    {
        GridView grid = (GridView)sender;
        grid.PageIndex = e.NewPageIndex;
        BindLocations();
        BindManagers();
    }

    protected void gridLocations_Sorting(object sender, GridViewSortEventArgs e)
    {
        BindLocations(gridLocations.GetSortDirection(), e.SortExpression);
        BindManagers();
    }

    protected void gridLocations_RowEditing(object sender, GridViewEditEventArgs e)
    {
        GridView grid = (GridView)sender;
        grid.EditIndex = e.NewEditIndex;
        grid.SelectedIndex = grid.EditIndex;
        BindLocations();
        BindManagers();
        grid.SelectedRow.FocusAndSelect("txtLocationName");
    }

    protected void gridLocations_RowCancelingEdit(object sender, GridViewCancelEditEventArgs e)
    {
        GridView grid = (GridView)sender;
        grid.EditIndex = -1;
        BindLocations();
    }

    protected void gridLocations_RowDeleting(object sender, GridViewDeleteEventArgs e)
    {
		GridView grid = (GridView)sender;

		Location location = new Location();
		location.LocID = grid.DataKeys[e.RowIndex].Value.ToInt32();
		location.Delete();

		BindLocations();
		BindManagers();
    }

    protected void btnDeleteLocation_Click(object sender, EventArgs e)
    {
		try
		{
			Location location = new Location();
			location.LocID = gridLocations.DataKeys[ClickedRow(sender).RowIndex].Value.ToInt32();
			location.Delete();
			BindLocations();
			BindManagers();
		}
		catch (Exception ex)
		{
			ShowMessage(ex.Message);
			//Log.Add(ex, Profile.UserName);
		}
    }

    protected void gridLocations_RowUpdating(object sender, GridViewUpdateEventArgs e)
    {
        GridView grid = (GridView)sender;
        GridViewRow row = grid.Rows[e.RowIndex];

        Location location = new Location(grid.DataKeys[e.RowIndex].Value.ToInt32());
        location.LocationName = row.GetValueIn("txtLocationName");
        location.DivID = row.GetValueIn("ddlEditDivision").ToInt32();
        location.Store();
        grid.EditIndex = -1;
        BindLocations();
        BindManagers();
    }

    protected void btnAddLocation_Click(object sender, EventArgs e)
    {
		try
		{
			Location location = new Location();
			location.ClientCode = Session["ClientCode"].ToInt32();
			location.LocationName = txtAddLocation.Text;
			location.DivID = ddlDivisions.SelectedValue.ToInt32();
			location.Store();

			//reset form
			gridLocations.EditIndex = -1;
			txtSearchLocations.Text = "";
			txtAddLocation.Text = "";
			ddlDivisions.SelectedIndex = -1;
			ddlAssignLocation.Items.Clear();

			BindLocations();
			gridLocations.SelectPage(location.LocID.ToString());
			BindManagers();
		}
		catch (System.Data.Common.DbException de)
		{
			if (de.Message.ToLower().Contains("duplicate"))
				ShowMessage("That location already exists");
			else
				ShowMessage(de.Message);
			//Log.Add(de, Profile.UserName);
		}
		catch (Exception ex)
		{
			ShowMessage(ex.Message);
			//Log.Add(ex, Profile.UserName);
		}
    }

	private void ShowMessage(string message)
	{
		lblMessage.Text = message;
		lblMessage.Visible = true;
	}

    protected void gridLocations_RowDataBound(object sender, GridViewRowEventArgs e)
    {
        if (e.Row.RowType == DataControlRowType.DataRow)
        {
            if ((e.Row.RowState & DataControlRowState.Edit) == DataControlRowState.Edit)
            {
                DropDownList ddlDivisionName = (DropDownList)e.Row.FindControl("ddlEditDivision");
                Location location = (Location)e.Row.DataItem;
                ddlDivisionName.BindList(Divisions.For(_client), "DivName", "DivID");
                ddlDivisionName.SelectItem("", location.DivID.ToString());
            }
            else
            {
                for (int i = 0; i < e.Row.Cells.Count; i++)
                    e.Row.Cells[i].Attributes["onclick"] = this.Page.ClientScript.GetPostBackClientHyperlink((GridView)sender, "Select$" + e.Row.RowIndex);
            }
            
            if (((GridView)sender).EditIndex < 0)
            {
                e.Row.Attributes["onmouseover"] = "if (this.className != 'selectedrow') this.className='mouseover'";
                e.Row.Attributes["onmouseout"] = "if (this.className != 'selectedrow') this.className='mouseout'";
            }
        }
    }

    protected void gridLocations_SelectedIndexChanging(object sender, GridViewSelectEventArgs e)
    {
        GridView grid = (GridView)sender;
        GridViewRow row = grid.Rows[e.NewSelectedIndex];
        grid.SelectedIndex = e.NewSelectedIndex;
        
        rdoClients.Checked = false;
        rdoDivisions.Checked = false;
        rdoLocations.Checked = true;
        BindManagers();
    }

    protected void btnShowAllLocations_Click(object sender, EventArgs e)
    {
		try
		{
			gridDivisions.SelectedIndex = -1;
			BindDivisions();
			BindLocations();
			BindManagers();
		}
		catch (Exception ex)
		{
			ShowMessage(ex.Message);
			//Log.Add(ex, Profile.UserName);
		}
    }

    #endregion

    #region gridManagers

    protected void rdoAssignedTo_CheckedChanged(object sender, EventArgs e)
    {
        BindManagers();
    }

    protected void btnSearchManagers_Click(object sender, EventArgs e)
    {
        gridManagers.SelectedIndex = -1;
        BindManagers();
    }

    protected void gridManagers_PageIndexChanging(object sender, GridViewPageEventArgs e)
    {
        GridView grid = (GridView)sender;
        grid.PageIndex = e.NewPageIndex;
        BindManagers();
    }

    protected void gridManagers_RowEditing(object sender, GridViewEditEventArgs e)
    {
        GridView grid = (GridView)sender;
        grid.EditIndex = e.NewEditIndex;
        grid.SelectedIndex = grid.EditIndex;
        BindManagers();
        //grid.SelectedRow.FocusAndSelect("txtFirstName");
    }

    protected void gridManagers_RowCancelingEdit(object sender, GridViewCancelEditEventArgs e)
    {
        GridView grid = (GridView)sender;
        grid.EditIndex = -1;
        BindManagers();
    }

    protected void gridManagers_RowDeleting(object sender, GridViewDeleteEventArgs e)
    {
        GridView grid = (GridView)sender;

        Manager m = new Manager();
        m.ManagerID = grid.DataKeys[e.RowIndex].Value.ToInt32();
        Database.Delete(m);

        BindManagers();
    }

    protected void gridManagers_RowUpdating(object sender, GridViewUpdateEventArgs e)
    {
        GridView grid = (GridView)sender;
        GridViewRow row = grid.Rows[e.RowIndex];

        Manager manager = new Manager(grid.DataKeys[e.RowIndex].Value.ToInt32());
        manager.Title = ((TextBox)row.Cells[grid.GetColumnIndex("Title")].Controls[0]).Text;
        manager.FirstName = ((TextBox)row.Cells[grid.GetColumnIndex("First Name")].Controls[0]).Text;
        manager.MiddleName = ((TextBox)row.Cells[grid.GetColumnIndex("Middle Name")].Controls[0]).Text;
        manager.LastName = ((TextBox)row.Cells[grid.GetColumnIndex("Last Name")].Controls[0]).Text;
        manager.Phone = ((TextBox)row.Cells[grid.GetColumnIndex("Phone")].Controls[0]).Text;
        manager.Extension = ((TextBox)row.Cells[grid.GetColumnIndex("Extension")].Controls[0]).Text;
        manager.Email = ((TextBox)row.Cells[grid.GetColumnIndex("Email")].Controls[0]).Text;
        manager.Store();

        grid.EditIndex = -1;
        BindManagers();
    }

    protected void btnAddManager_Click(object sender, EventArgs e)
    {
		try
		{
			txtSearchManagers.Text = "";

			dvManager.ChangeMode(DetailsViewMode.Edit);

			dvManager.DataSource = new List<Manager> { new Manager() };
			dvManager.DataBind();

			Page.SetFocus(btnPageBottom);
		}
		catch (Exception ex)
		{
			ShowMessage(ex.Message);
			//Log.Add(ex, Profile.UserName);
		}
    }

    protected void btnDeleteManager_Click(object sender, EventArgs e)
    {
		try
		{
            Manager m = new Manager();
            m.ManagerID = gridManagers.SelectedDataKey["ManagerID"].ToInt32();
            Database.Delete(m);

			dvManager.ChangeMode(DetailsViewMode.ReadOnly);
			BindManagers();
		}
		catch (Exception ex)
		{
			//Log.Add(ex, Profile.UserName);
			ShowMessage(ex.Message);
		}
    }

    protected void gridManagers_RowDataBound(object sender, GridViewRowEventArgs e)
    {
        if (e.Row.RowType == DataControlRowType.DataRow)
        {
            if ((e.Row.RowState & DataControlRowState.Edit) == DataControlRowState.Edit)
            {
            }
            else
            {
                for (int i = 0; i < e.Row.Cells.Count; i++)
                    e.Row.Cells[i].Attributes["onclick"] = this.Page.ClientScript.GetPostBackClientHyperlink((GridView)sender, "Select$" + e.Row.RowIndex);
            }

            if (((GridView)sender).EditIndex < 0)
            {
                e.Row.Attributes["onmouseover"] = "if (this.className != 'selectedrow') this.className='mouseover';";
                e.Row.Attributes["onmouseout"] = "if (this.className != 'selectedrow') this.className='mouseout';";
            }
        }
    }

    protected void gridManagers_SelectedIndexChanging(object sender, GridViewSelectEventArgs e)
    {
        GridView grid = (GridView)sender;
        GridViewRow row = grid.Rows[e.NewSelectedIndex];
        gridManagers.SelectedIndex = e.NewSelectedIndex;

        dvManager.ChangeMode(DetailsViewMode.Edit);

        BindManager();
        BindAssignedDivisions();
        BindAssignedLocations();

        Page.SetFocus(btnPageBottom);
    }

    protected void btnShowAllManagers_Click(object sender, EventArgs e)
    {
		try
		{
			rdoDivisions.Checked = false;
			rdoLocations.Checked = false;
			rdoClients.Checked = true;

			txtSearchManagers.Text = "";
			gridManagers.SelectedIndex = -1;
			BindManagers();
		}
		catch (Exception ex)
		{
			ShowMessage(ex.Message);
			//Log.Add(ex, Profile.UserName);
		}
    }

    private GridViewRow ClickedRow(object sender)
    {
        GridViewRow clickedRow = null;
        if (sender is LinkButton)
            clickedRow = ((LinkButton)sender).NamingContainer as GridViewRow;
        else if (sender is Button)
            clickedRow = ((Button)sender).NamingContainer as GridViewRow;
        else if (sender is CheckBox)
            clickedRow = ((CheckBox)sender).NamingContainer as GridViewRow;
        else if (sender is Control)
            clickedRow = ((Control)sender).NamingContainer as GridViewRow;
        else
            throw new NotSupportedException(String.Format("The sender {0} is not yet supported.", sender.GetType().Name));

        if (clickedRow == null)
            throw new InvalidOperationException("Clicked row is null.");

        return clickedRow;
    }

    #endregion

    #region Assign Division

    protected void gvAssignedDivisions_SelectedIndexChanging(object sender, GridViewSelectEventArgs e)
    {
        GridView grid = (GridView)sender;
        GridViewRow row = grid.Rows[e.NewSelectedIndex];
        grid.SelectedIndex = e.NewSelectedIndex;
    }

    protected void gvAssignedDivisions_RowDataBound(object sender, GridViewRowEventArgs e)
    {
        if (e.Row.RowType == DataControlRowType.DataRow)
        {
            if ((e.Row.RowState & DataControlRowState.Edit) == DataControlRowState.Edit)
            {
            }
            else
            {
                for (int i = 0; i < e.Row.Cells.Count; i++)
                    e.Row.Cells[i].Attributes["onclick"] = this.Page.ClientScript.GetPostBackClientHyperlink((GridView)sender, "Select$" + e.Row.RowIndex);
            }

            if (((GridView)sender).EditIndex < 0)
            {
                e.Row.Attributes["onmouseover"] = "if (this.className != 'selectedrow') this.className='mouseover'";
                e.Row.Attributes["onmouseout"] = "if (this.className != 'selectedrow') this.className='mouseout'";
            }
        }
    }

    protected void gvAssignedDivisions_RowDeleting(object sender, GridViewDeleteEventArgs e)
    {
        GridView grid = (GridView)sender;

        int divisionID = grid.DataKeys[e.RowIndex].Values[0].ToInt32();
        int managerID = grid.DataKeys[e.RowIndex].Values[1].ToInt32();

        Manager.UnassignFromDivision(divisionID, managerID);

        BindAssignedDivisions();
    }

    protected void gvAssignedDivisions_PageIndexChanging(object sender, GridViewPageEventArgs e)
    {
        GridView grid = (GridView)sender;
        grid.PageIndex = e.NewPageIndex;
        BindAssignedDivisions();
    }

    protected void chkDivisionIsPrimary_CheckedChanged(object sender, EventArgs e)
    {
        gvAssignedDivisions.SelectedIndex = ClickedRow(sender).RowIndex;
        
        int managerID = gridManagers.SelectedDataKey.Value.ToInt32();
        int divisionID = gvAssignedDivisions.DataKeys[gvAssignedDivisions.SelectedIndex].Values[0].ToInt32();
        bool primary = ((CheckBox)sender).Checked;

        Manager.AssignToDivision(divisionID, managerID, primary);
        
        BindAssignedDivisions();
    }

    protected void btnAssignDivision_Click(object sender, EventArgs e)
    {
        if (gridManagers.SelectedIndex < 0)
            return;

        int managerID = gridManagers.SelectedDataKey.Value.ToInt32();
        int divisionID = ddlAssignDivision.SelectedValue.ToInt32();

        Manager.AssignToDivision(divisionID, managerID, chkDivisionIsPrimary.Checked);
        
        BindAssignedDivisions();

        ddlAssignDivision.SelectedIndex = 0;
        chkDivisionIsPrimary.Checked = false;
    }

    #endregion

    #region Assign Location

    protected void gvAssignedLocations_SelectedIndexChanging(object sender, GridViewSelectEventArgs e)
    {
        GridView grid = (GridView)sender;
        GridViewRow row = grid.Rows[e.NewSelectedIndex];
        grid.SelectedIndex = e.NewSelectedIndex;
    }

    protected void gvAssignedLocations_RowDataBound(object sender, GridViewRowEventArgs e)
    {
        if (e.Row.RowType == DataControlRowType.DataRow)
        {
            if ((e.Row.RowState & DataControlRowState.Edit) == DataControlRowState.Edit)
            {
            }
            else
            {
                for (int i = 0; i < e.Row.Cells.Count; i++)
                    e.Row.Cells[i].Attributes["onclick"] = this.Page.ClientScript.GetPostBackClientHyperlink((GridView)sender, "Select$" + e.Row.RowIndex);
            }

            if (((GridView)sender).EditIndex < 0)
            {
                e.Row.Attributes["onmouseover"] = "if (this.className != 'selectedrow') this.className='mouseover'";
                e.Row.Attributes["onmouseout"] = "if (this.className != 'selectedrow') this.className='mouseout'";
            }
        }
    }

    protected void gvAssignedLocations_RowDeleting(object sender, GridViewDeleteEventArgs e)
    {
        GridView grid = (GridView)sender;

        int locationID = grid.DataKeys[e.RowIndex].Values[0].ToInt32();
        int managerID = grid.DataKeys[e.RowIndex].Values[1].ToInt32();

        Manager.UnassignFromLocation(locationID, managerID);

        BindAssignedLocations();
    }

    protected void gvAssignedLocations_PageIndexChanging(object sender, GridViewPageEventArgs e)
    {
        GridView grid = (GridView)sender;
        grid.PageIndex = e.NewPageIndex;
        BindAssignedLocations();
    }

    protected void chkLocationIsPrimary_CheckedChanged(object sender, EventArgs e)
    {
        gvAssignedLocations.SelectedIndex = ClickedRow(sender).RowIndex;

        int managerID = gridManagers.SelectedDataKey.Value.ToInt32();
        int locationID = gvAssignedLocations.DataKeys[gvAssignedLocations.SelectedIndex].Values[0].ToInt32();
        bool primary = ((CheckBox)sender).Checked;

        Manager.AssignToLocation(locationID, managerID, primary);

        BindAssignedLocations();
    }

    protected void btnAssignLocation_Click(object sender, EventArgs e)
    {
        if (gridManagers.SelectedIndex < 0)
            return;

        int managerID = gridManagers.SelectedDataKey.Value.ToInt32();
        int locationID = ddlAssignLocation.SelectedValue.ToInt32();

        Manager.AssignToLocation(locationID, managerID, chkLocationIsPrimary.Checked);

        BindAssignedLocations();

        ddlAssignLocation.SelectedIndex = 0;
        chkLocationIsPrimary.Checked = false;
    }

    #endregion

    #region Manager DetailsView

    protected void dvManager_ModeChanging(object sender, DetailsViewModeEventArgs e)
    {
        Manager m = null;

        if (e.NewMode == DetailsViewMode.Insert)
            m = new Manager();
        else if (e.NewMode == DetailsViewMode.ReadOnly)
            m = null;
        else if (gridManagers.SelectedIndex >= 0)
            m = new Manager(gridManagers.SelectedDataKey.Value.ToInt32());

        dvManager.ChangeMode(e.NewMode);

        if (m != null)
            dvManager.DataSource = new List<Manager> { m };
        else
            dvManager.DataSource = null;
        dvManager.DataBind();
    }

    protected void dvManager_ItemUpdating(object sender, EventArgs e)
    {
        Manager m = SaveManager();
        dvManager.ChangeMode(DetailsViewMode.ReadOnly);

        BindManagers();
        gridManagers.SelectPage(m.ManagerID.ToString());

        //rebind the associated view and grids after selecting page/row
        BindManager();
        BindAssignedDivisions();
        BindAssignedLocations();
    }

    private Manager SaveManager()
    {
        Manager m = null;

        int managerID = dvManager.DataKey["ManagerID"].ToInt32();
        if (managerID > 0)
            m = new Manager(managerID);
        else
            m = new Manager();

        m.ClientCode = Session["ClientCode"].ToInt32();
        m.Title = ((TextBox)dvManager.Rows[0].FindControl("txtTitle")).Text;
        m.FirstName = ((TextBox)dvManager.Rows[1].FindControl("txtFirstName")).Text;
        m.MiddleName = ((TextBox)dvManager.Rows[2].FindControl("txtMiddleName")).Text;
        m.LastName = ((TextBox)dvManager.Rows[3].FindControl("txtLastName")).Text;
        m.Phone = ((TextBox)dvManager.Rows[4].FindControl("txtPhone")).Text;
        m.Extension = ((TextBox)dvManager.Rows[5].FindControl("txtExtension")).Text;
        m.Email = ((TextBox)dvManager.Rows[6].FindControl("txtEmail")).Text;
        m.Store();

        if (gridDivisions.SelectedIndex >= 0 && gridDivisions.SelectedDataKey != null)
            m.AssignToDivision(gridDivisions.SelectedDataKey.Value.ToInt32(), true);
        
        if (gridLocations.SelectedIndex >= 0 && gridLocations.SelectedDataKey != null)
            m.AssignToLocation(gridLocations.SelectedDataKey.Value.ToInt32(), true);

        return m;
    }

    #endregion

	protected override void OnError(EventArgs e)
	{
		HttpContext ctx = HttpContext.Current;

		Exception exception = ctx.Server.GetLastError();

		//Log.Add(exception, Profile.UserName);
		ShowMessage(exception.Message);

		ctx.Server.ClearError();

		base.OnError(e);
	}
}


    //protected void dvManager_ItemCreated(object sender, EventArgs e)
    //{
    //    if (dvManager.FooterRow != null)
    //    {
    //        int commandRowIndex = dvManager.Rows.Count - 1;
    //        if (commandRowIndex > 0)
    //        {
    //            DataControlFieldCell cell = (DataControlFieldCell)dvManager.Rows[commandRowIndex].Controls[0];
    //            foreach (Control c in cell.Controls)
    //            {
    //                Button button = c as Button;
    //                if (button != null && button.CommandName == "Delete")
    //                    button.OnClientClick = "javascript:if (!confirm('Do you really want to delete this manager?')) return false;";
    //            }
    //        }
    //    }
    //}
